@@ -96,6 +96,23 @@ gdb_breakpoint_set_watchpoint (GdbInstance *inst,
     return read_new_watchpoint (inst);
 }
 
+static gint
+all_breakpoint_traverse_func (gpointer key, gpointer value, gpointer data)
+{
+    GList **list = data;
+    *list = g_list_prepend (*list, value);
+    return FALSE;
+}
+
+GList *
+gdb_breakpoint_get_all (GdbInstance *inst)
+{
+    GList *list = NULL;
+    g_tree_traverse (inst->breakpoints, all_breakpoint_traverse_func,
+		     G_IN_ORDER, &list);
+    return g_list_reverse (list);
+}
+
 const GdbBreakpoint *
 gdb_breakpoint_get_info (GdbInstance *inst, int bp_num)
 {
