@@ -61,31 +61,24 @@ gdb_get_locals (GdbInstance *inst, GdfLibdryadSymbolSet *ss)
 	CORBA_free (ss->symbols);
     }
     ss->set_size = 0;
-    
-    gdb_execute_command (inst, "info locals");
-
     ss->symbols = gdb_allocate_symbols ();
 
-    /* Read the names of all the locals */
+    gdb_execute_command (inst, "info scope *($pc)");
+
+    /* Look for the scope header */
     line = gdb_read_line (inst);
-    while (strcmp (line, "\32\32pre-prompt")) {
-	if ((work = strstr (line, " = "))) {
-	    *work = '\0';
-	    work += 3;
-	    
-	    names = g_list_prepend (names, g_strdup (line));
-	    
-	    if (!strncmp (work, "{", 1)) {
-		int levels = 1;
-		while (levels > 0) {
-		    line = gdb_read_line (inst);
-		    if (!strncmp (line, "{", 1) || !strncmp (line, ", {", 3))
-			levels++;
-		    if (!strncmp (line, "}", 1) || !strncmp (line, "...}", 4))
-			levels--;
-		}
-	    }
-	}
+    while (strncmp (line, "Scope for *($pc)", strlen ("Scope for *($pc)")))
+	line = gdb_read_line (inst);
+
+    /* read all the variables in the scope */
+    line = gdb_read_line (inst);
+    while (!isspace (line[0]) && line[0] != '\0') {
+	line += strlen ("Symbol ");
+	work = line;
+	while (!isspace (*work++))
+	    ;
+	*(work - 1) = '\0';
+	names = g_list_prepend (names, g_strdup (line));
 	line = gdb_read_line (inst);
     }
 
@@ -299,6 +292,7 @@ void
 read_type (GdbInstance *inst, GDF_Symbol *sym)
 {
     char *line;
+    char *p;
     char *cmd_str;
 
     g_assert (sym->expression);
@@ -322,6 +316,8 @@ read_type (GdbInstance *inst, GDF_Symbol *sym)
 
 	    type = g_strdup (line);
 
+	    /* If the type contains a structure, skip over the structure
+	     * definition and append anything after that. */
 	    if (line[strlen (line) - 1] == '{') {
 		int depth = 1;
 		char *newtype;
@@ -341,6 +337,17 @@ read_type (GdbInstance *inst, GDF_Symbol *sym)
 		type = newtype;
 	    }
 
+	    /* End the type at a colon (C++ inheritance operator) */
+	    p = strchr (type, ':');
+	    if (p)
+		*p = '\0';
+
+	    /* Remove trailing whitespace */
+	    p = type + strlen(type);
+	    while (isspace (*p--))
+		;
+	    *(p + 1) = '\0';
+	    
 	    sym->typename = CORBA_string_dup (type);
 	    g_free (type);
 	    
@@ -353,7 +360,8 @@ read_type (GdbInstance *inst, GDF_Symbol *sym)
     
     sym->expandable = FALSE;
     sym->type = GDF_TYPE_NORMAL;
-    if (!strncmp (sym->typename, "struct", strlen ("struct"))) {
+    if (!strncmp (sym->typename, "struct", strlen ("struct"))
+	|| !strncmp (sym->typename, "class", strlen ("class"))) {
 	sym->expandable = TRUE;
 	sym->type = GDF_TYPE_STRUCT;
     }
@@ -464,6 +472,41 @@ expand_pointer (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
     g_free (new_name);
 }
 
+static char *
+get_child_line (GdbInstance *inst)
+{
+    char *work;
+    char *line = gdb_read_line (inst);
+
+    while (1) {
+	/* When debugging C++, gdb seperates data values and function values 
+	 * with an empty line.  Look for that empty line to end processing.
+	 * At some point we could put in support for function values, but I'm 
+	 * to lazy to do that just yet */
+	for (work = line; isspace (*work); work++)
+	    ;
+	if (*work == '\0')
+	    return NULL;
+
+	if (line[0] == '}') 
+	    return NULL;
+
+	/* Access specifiers begin in column 3, whereas variables begin in
+	 * column 5.  If it is an access specifier we try the next line, 
+	 * otherwise we return this line IF the line is not a static member
+	 * of a C++ class. */
+	g_assert (strlen (line) > 2);
+	if (line[2] == ' ' 
+	    && line[4] != ' '
+	    && strncmp (line + 4, "static", strlen ("static"))) {
+	    return line;
+	}
+
+
+	line = gdb_read_line (inst);
+    }
+}
+
 static char *
 read_child_from_ptype (GdbInstance *inst)
 {
@@ -471,10 +514,10 @@ read_child_from_ptype (GdbInstance *inst)
     char *work;
     char *line;
 
-    line = gdb_read_line (inst);
-
-    if (line[0] == '}') 
+    line = get_child_line (inst);
+    if (!line) {
 	return NULL;
+    }
 
     /* Clean out the trailing ;, and any array indices if necessary */
     work = line + (strlen (line) - 1);
@@ -500,12 +543,83 @@ read_child_from_ptype (GdbInstance *inst)
 	*(work + 1) = '\0';
     }
 
-    while (*work != '*' && *work-- != ' ')
+    while (*work != '*' && *work-- != ' ' && work != line)
 	;
     name = work + 1;
     return g_strdup (name);
 }
 
+static GList *
+get_ptype_names (GdbInstance *inst, char *type_line, 
+		 GList *names, GTree *existing_names)
+{
+    static char *derive_keywords[] = { "public", "private", "protected",
+				       "virtual" };
+    char *line;
+    char *type_line_copy = g_strdup (type_line + strlen ("type = "));
+    char *p;
+
+    char *name;
+    
+    name = read_child_from_ptype (inst);
+    while (name) {
+	if (!g_tree_lookup (existing_names, name)) {
+	    names = g_list_prepend (names, name);
+	    g_tree_insert (existing_names, name, name);
+	}
+	name = read_child_from_ptype (inst);
+    }
+
+    /* Find parent classes and get_ptype_names from them */
+    p = strchr (type_line_copy, '{');
+    if (p)
+	*p = '\0';
+    p = strchr (type_line_copy, ':');
+    if (p)
+	p += 2;
+    while (p && *p) {
+	char *type;
+	int i;
+	gboolean is_keyword = FALSE;
+	
+	while (*p && isspace (*p))
+	    p++;
+	type = p;
+	
+	while (*p && !isspace (*p) && *p != ',') 
+	    p++;
+	if (*p == '\0')
+	    p = NULL;
+	else 
+	    *p = '\0';
+
+	for (i = 0; i < sizeof (derive_keywords) / sizeof (derive_keywords[0]); i++) {
+	    if (!strcmp (type, derive_keywords[i])) {
+		is_keyword = TRUE;
+		break;
+	    }
+	} 
+	
+	if (!is_keyword) {
+	    char *cmd_str = g_strdup_printf ("ptype %s", type);
+	    gdb_execute_command (inst, cmd_str);
+	    g_free (cmd_str);
+	    
+	    do {
+		line = gdb_read_line (inst);
+	    } while (strncmp (line, "type = ", strlen ("type = ")));
+	    
+	    names = get_ptype_names (inst, line, names, existing_names);
+	}
+	if (p)
+	    p++;
+    }
+    
+    g_free (type_line_copy);
+	    
+    return names;
+}
+
 void
 expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss, 
 		  GDF_Symbol *sym)
@@ -513,8 +627,8 @@ expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
     char *expr;
     char *cmd_str;
     char *line;
-    char *name;
     GList *names = NULL;
+    GTree *existing_names;
     GList *i;
 
     cmd_str = g_strdup_printf ("ptype %s", sym->expression);
@@ -525,11 +639,8 @@ expand_structure (GdbInstance *inst, GdfLibdryadSymbolSet *ss,
 	line = gdb_read_line (inst);
     } while (strncmp (line, "type = ", strlen ("type = ")));
 
-    name = read_child_from_ptype (inst);
-    while (name) {
-	names = g_list_prepend (names, name);
-	name = read_child_from_ptype (inst);
-    }
+    existing_names = g_tree_new ((GCompareFunc)strcmp);
+    names = get_ptype_names (inst, line, names, existing_names);
 
     names = g_list_reverse (names);
     for (i = names; i != NULL; i = i->next) {
