@@ -238,7 +238,6 @@ impl_attach (PortableServer_Servant servant,
     switch (res) {
     case GDB_ATTACH_OK :
 	dbg->state |= GDF_Debugger_ATTACHED;
-	dbg->state |= GDF_Debugger_TARGET_EXISTS;
 	event_any = gdf_marshal_event_none ("attached");
 	gdf_event_channel_client_push (dbg->event_channel, event_any);
 	CORBA_free (event_any);
@@ -269,7 +268,7 @@ impl_detach (PortableServer_Servant servant,
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
     CORBA_any *event_any;
 
-    if ((dbg->state & GDF_Debugger_ATTACHED) != GDF_Debugger_ATTACHED) {
+    if (!(dbg->state & GDF_Debugger_ATTACHED)) {
 	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
 			     ex_GDF_Debugger_InvalidState, NULL);
 	return;
@@ -290,13 +289,21 @@ impl_detach (PortableServer_Servant servant,
 
 static void
 impl_execute (PortableServer_Servant servant, 
-	      const GDF_arg_list *args,
+	      const CORBA_char *args,
 	      CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
     g_return_if_fail (dbg->gdb_inst != NULL);
 
-    gdb_run (dbg->gdb_inst);
+    if ((dbg->state & GDF_Debugger_TARGET_EXISTS) 
+	|| (dbg->state & GDF_Debugger_COREFILE)) {
+	
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
+    gdb_run (dbg->gdb_inst, args);
 }
 
 static void
@@ -304,7 +311,67 @@ impl_load_corefile (PortableServer_Servant servant,
 		    const CORBA_char *corefile_name,
 		    CORBA_Environment *ev)
 {
-    /* not implemented */
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    GdbCoreResult res;
+    CORBA_any *event_any;
+
+    if ((dbg->state & GDF_Debugger_TARGET_EXISTS)
+	|| (dbg->state & GDF_Debugger_COREFILE)) {
+	
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
+    res = gdb_load_core (dbg->gdb_inst, corefile_name);
+
+    switch (res) {
+    case GDB_CORE_OK :
+	dbg->state |= GDF_Debugger_COREFILE;
+	event_any = gdf_marshal_event_none ("corefile_loaded");
+	gdf_event_channel_client_push (dbg->event_channel, event_any);
+	CORBA_free (event_any);
+	break;
+    case GDB_CORE_ALREADY_RUNNING :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	break;
+    case GDB_CORE_DOESNT_EXIST :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_DoesntExist, NULL);
+	break;
+    case GDB_CORE_ACCESS_DENIED :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_SecurityException, NULL);
+	break;
+    case GDB_CORE_INVALID_COREFILE :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			     ex_GDF_Debugger_InvalidCorefile, NULL);
+	break;
+    case GDB_CORE_MISC :
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_UnknownError, NULL);
+	break;
+    }
+}
+
+static void
+impl_unload_corefile (PortableServer_Servant servant, CORBA_Environment *ev)
+{
+    GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+    CORBA_any *event_any;
+
+    if (!(dbg->state & GDF_Debugger_COREFILE)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
+    gdb_unload_core (dbg->gdb_inst);
+    dbg->state &= ~GDF_Debugger_COREFILE;
+    event_any = gdf_marshal_event_none ("corefile_unloaded");
+    gdf_event_channel_client_push (dbg->event_channel, event_any);
+    CORBA_free (event_any);
 }
 
 static GDF_source_file_list *
@@ -316,6 +383,12 @@ impl_get_sources (PortableServer_Servant servant, CORBA_Environment *ev)
     GList *i;
     int num;
 
+    if (dbg->state & GDF_Debugger_BINARY_LOADED) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return GDF_source_file_list__alloc (); 
+    }
+
     sources = gdb_list_sources (dbg->gdb_inst);    
 
     list = GDF_source_file_list__alloc ();
@@ -337,7 +410,14 @@ impl_get_absolute_source_path (PortableServer_Servant servant,
 			       CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    CORBA_char *retval;
+    CORBA_char *retval;    
+
+    if (!(dbg->state & GDF_Debugger_BINARY_LOADED)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return CORBA_string_dup (""); 
+    }
+
     
     if (source_file[0] == '/')
 	retval = CORBA_string_dup (source_file);
@@ -351,6 +431,43 @@ impl_get_absolute_source_path (PortableServer_Servant servant,
     return retval;
 }
 
+static GDF_BreakpointList *
+impl_get_breakpoints (PortableServer_Servant servant,
+		      CORBA_Environment *ev)
+{
+     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+     GList *breakpoints;
+     GList *i;
+     int n;
+     GDF_BreakpointList *ret = GDF_BreakpointList__alloc ();
+
+     breakpoints = gdb_breakpoint_get_all (dbg->gdb_inst);
+     n = g_list_length (breakpoints); 
+     ret->_length = n;
+     ret->_maximum = n;
+     ret->_buffer = CORBA_sequence_GDF_Breakpoint_allocbuf (n);
+     CORBA_sequence_set_release (ret, TRUE);
+
+     n = 0;
+     for (i = breakpoints; i != NULL; i = i->next) {
+	 GdbBreakpoint *bp = i->data;
+	 GDF_Breakpoint *corba_bp = &ret->_buffer[n++];
+
+	 corba_bp->num = bp->num;
+	 corba_bp->type = CORBA_string_dup (bp->type ? bp->type : "");
+	 corba_bp->enabled = (CORBA_boolean)bp->enabled;
+	 corba_bp->address = CORBA_string_dup (bp->address ? bp->address : "");
+	 corba_bp->file_name = 
+	     CORBA_string_dup (bp->file_name ? bp->file_name : "");
+	 corba_bp->line_num = (CORBA_long)bp->line_num;	 
+     }
+     
+     g_list_free (breakpoints);
+
+     return ret;
+     
+}
+
 static CORBA_long
 impl_set_breakpoint (PortableServer_Servant servant, 
 		     const CORBA_char *file_name,
@@ -513,7 +630,7 @@ impl_get_breakpoint_info (PortableServer_Servant servant,
     bp = gdb_breakpoint_get_info (dbg->gdb_inst, bp_num);
     
     ret = GDF_Breakpoint__alloc ();
-
+    
     if (bp == NULL) {
 	/* FIXME: Set an exception */
 	return ret;
@@ -540,8 +657,16 @@ impl_get_registers (PortableServer_Servant servant,
     int num_regs = 0;
     GList *r;
     int i = 0;
-    
+
     ret = GDF_RegisterList__alloc ();
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)
+	&& !(dbg->state & GDF_Debugger_COREFILE)) {
+	
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return ret;
+    }
+
     CORBA_sequence_set_release (ret, CORBA_TRUE);
 
     if (dbg->gdb_inst) {
@@ -568,7 +693,7 @@ impl_get_registers (PortableServer_Servant servant,
     
     /* Did not get registers; must not be running. */
     CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
-			 ex_GDF_Debugger_NotRunning, NULL);
+			 ex_GDF_Debugger_InvalidState, NULL);
 
     return ret;
 }
@@ -579,7 +704,15 @@ impl_change_frame (PortableServer_Servant servant,
 		   CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-    
+
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)
+	&& !(dbg->state & GDF_Debugger_COREFILE)) {
+	
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     if (gdb_change_frame (dbg->gdb_inst, (int) new_id)) {
 	CORBA_any *event_any;
 	process_frame_change (dbg);
@@ -601,7 +734,8 @@ impl_get_frame (PortableServer_Servant servant,
 
     gdf_frame = GDF_StackFrame__alloc ();
 
-    if ((dbg->state & GDF_Debugger_TARGET_EXISTS) == GDF_Debugger_TARGET_EXISTS) {
+    if ((dbg->state & GDF_Debugger_TARGET_EXISTS)
+	|| (dbg->state & GDF_Debugger_COREFILE)) {
 	frame = gdb_get_frame (dbg->gdb_inst, id);
 	
 	gdf_frame->id = frame->id;
@@ -625,7 +759,7 @@ impl_get_frame (PortableServer_Servant servant,
 	gdf_frame->virtual_address = 0;
 	
 	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-			     ex_GDF_Debugger_NotRunning, NULL);
+			     ex_GDF_Debugger_InvalidState, NULL);
     }
     
     return gdf_frame;
@@ -641,14 +775,17 @@ impl_get_backtrace (PortableServer_Servant servant, CORBA_Environment *ev)
     int n;
 
     stack = GDF_Stack__alloc ();
-    CORBA_sequence_set_release (stack, CORBA_TRUE);
 
-    if ((dbg->state & GDF_Debugger_TARGET_EXISTS) != GDF_Debugger_TARGET_EXISTS) {
-	CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
-			     ex_GDF_Debugger_NotRunning, NULL);
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)
+	&& !(dbg->state & GDF_Debugger_COREFILE)) {
+	
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
 	return stack;
     }
 
+    CORBA_sequence_set_release (stack, CORBA_TRUE);
+
     list = gdb_get_stack (dbg->gdb_inst);    
 
     stack->_length = stack->_maximum = g_list_length (list);
@@ -672,9 +809,16 @@ static void
 impl_cont (PortableServer_Servant servant, CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
-
+    
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
+
     gdb_continue (dbg->gdb_inst);    
 }
 
@@ -685,6 +829,12 @@ impl_stop (PortableServer_Servant servant, CORBA_Environment *ev)
 
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     gdb_stop (dbg->gdb_inst);
 }
 
@@ -695,8 +845,14 @@ impl_restart (PortableServer_Servant servant, CORBA_Environment *ev)
 
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     gdb_stop (dbg->gdb_inst);
-    gdb_run (dbg->gdb_inst);
+    gdb_run (dbg->gdb_inst, NULL);
 }
 
 static void
@@ -706,6 +862,12 @@ impl_step_over (PortableServer_Servant servant, CORBA_Environment *ev)
 
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     gdb_step_over (dbg->gdb_inst);
 }
 
@@ -716,6 +878,12 @@ impl_step_into (PortableServer_Servant servant, CORBA_Environment *ev)
 
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     gdb_step_into (dbg->gdb_inst);
 }
 
@@ -726,6 +894,12 @@ impl_step_out (PortableServer_Servant servant, CORBA_Environment *ev)
 
     g_return_if_fail (dbg->gdb_inst != NULL);
 
+    if (!(dbg->state & GDF_Debugger_TARGET_EXISTS)) {
+	CORBA_exception_set (ev, CORBA_USER_EXCEPTION, 
+			     ex_GDF_Debugger_InvalidState, NULL);
+	return;
+    }
+
     gdb_step_out (dbg->gdb_inst);
 }
 
@@ -756,6 +930,7 @@ static GDF_SymbolSet
 impl_get_locals (PortableServer_Servant servant, CORBA_Environment *ev)
 {
     GdfLibdryadDebugger *dbg = libdryad_debugger_from_servant (servant);
+
     return CORBA_Object_duplicate (bonobo_object_corba_objref (BONOBO_OBJECT (dbg->locals)), 
 				   ev);
 }
@@ -810,8 +985,10 @@ init_libdryad_debugger_corba_class (void)
     libdryad_debugger_epv.detach = impl_detach;
     libdryad_debugger_epv.execute = impl_execute;
     libdryad_debugger_epv.load_corefile = impl_load_corefile;
+    libdryad_debugger_epv.unload_corefile = impl_unload_corefile;
     libdryad_debugger_epv.get_sources = impl_get_sources;
     libdryad_debugger_epv.get_absolute_source_path = impl_get_absolute_source_path;
+    libdryad_debugger_epv.get_breakpoints = impl_get_breakpoints;    
     libdryad_debugger_epv.set_breakpoint = impl_set_breakpoint;
     libdryad_debugger_epv.set_breakpoint_function = impl_set_breakpoint_function;
     libdryad_debugger_epv.set_watchpoint = impl_set_watchpoint;
@@ -870,22 +1047,27 @@ static void
 execution_starting_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
-    dbg->state |= GDF_Debugger_TARGET_EXISTS;
-
-    event_any = gdf_marshal_event_none ("started");
-    gdf_event_channel_client_push (dbg->event_channel, event_any);
-    CORBA_free (event_any);
+    
+    if (!dbg->gdb_inst->corefile_loaded) {
+	dbg->state |= GDF_Debugger_TARGET_EXISTS;
+	event_any = gdf_marshal_event_none ("started");
+	gdf_event_channel_client_push (dbg->event_channel, event_any);
+	CORBA_free (event_any);
+    } else {
+	dbg->state |= GDF_Debugger_COREFILE;
+    }
 }
 
 static void
 execution_running_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
-    dbg->state |= GDF_Debugger_TARGET_EXECUTING;
-
-    event_any = gdf_marshal_event_none ("running");
-    gdf_event_channel_client_push (dbg->event_channel, event_any);
-    CORBA_free (event_any);
+    if (!dbg->gdb_inst->corefile_loaded) {
+	dbg->state |= GDF_Debugger_TARGET_EXECUTING;
+	event_any = gdf_marshal_event_none ("running");
+	gdf_event_channel_client_push (dbg->event_channel, event_any);
+	CORBA_free (event_any);
+    }
 }
 
 static void
@@ -929,9 +1111,11 @@ execution_stopped_cb (GdfLibdryadDebugger *dbg)
 {
     CORBA_any *event_any;
     dbg->state &= ~GDF_Debugger_TARGET_EXECUTING;
-
-    event_any = gdf_marshal_event_none ("stopped");
-    dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
+    
+    if (!dbg->gdb_inst->corefile_loaded) {
+	event_any = gdf_marshal_event_none ("stopped");
+	dbg->event_queue = g_list_prepend (dbg->event_queue, event_any);
+    }
 
     if (dbg->pending_exit != -1) {
 	dbg->state &= ~GDF_Debugger_TARGET_EXISTS;
