@@ -47,6 +47,7 @@ struct _GdfSourceViewerManagerPriv {
 	gint breakpoint_enabled_sig;
 	gint breakpoint_disabled_sig;
 	gint breakpoint_deleted_sig;
+    gint corefile_unloaded_sig;
 	gint execution_killed_sig;
 	gint execution_exited_sig;
 	gint execution_source_line_sig;
@@ -110,6 +111,8 @@ static void breakpoint_deleted_cb (GdfDebuggerClient *dbg,
 static void execution_source_line_cb (GdfDebuggerClient *dbg, 
 									  gchar *file, gint line, 
 									  GdfSourceViewerManager *svm);
+static void corefile_unloaded_cb (GdfDebuggerClient *dbg,
+                                  GdfSourceViewerManager *svm);
 static void execution_exited_cb (GdfDebuggerClient *dbg,
 								 gint exit_code, GdfSourceViewerManager *svm);
 static void frame_changed_cb (GdfDebuggerClient *dbg, gint new_frame,
@@ -390,8 +393,10 @@ check_sources_entry (GdfSourceViewerManager *svm, gchar *filename)
     if (strcmp (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (svm->priv->sources)->entry)),
                 text) != 0) {
         
-        gtk_signal_disconnect (GTK_OBJECT (GTK_COMBO (svm->priv->sources)->entry),
-                               svm->priv->sources_changed_sig);
+        if (svm->priv->sources_changed_sig != -1) {
+            gtk_signal_disconnect (GTK_OBJECT (GTK_COMBO (svm->priv->sources)->entry),
+                                   svm->priv->sources_changed_sig);
+        }
         
         gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (svm->priv->sources)->entry),
                             text);
@@ -718,6 +723,11 @@ connect_debugger_signals (GdfSourceViewerManager *svm)
 							"breakpoint_deleted",
 							GTK_SIGNAL_FUNC (breakpoint_deleted_cb),
 							(gpointer)svm);
+	svm->priv->corefile_unloaded_sig = 
+		gtk_signal_connect (GTK_OBJECT (svm->priv->dbg),
+							"corefile_unloaded",
+							GTK_SIGNAL_FUNC (corefile_unloaded_cb),
+							(gpointer)svm);
 	svm->priv->execution_source_line_sig = 
 		gtk_signal_connect (GTK_OBJECT (svm->priv->dbg),
 							"execution_source_line",
@@ -752,6 +762,8 @@ disconnect_debugger_signals (GdfSourceViewerManager *svm)
 						   svm->priv->breakpoint_deleted_sig);
 	gtk_signal_disconnect (GTK_OBJECT (svm->priv->dbg), 
 						   svm->priv->execution_source_line_sig);
+    gtk_signal_disconnect (GTK_OBJECT (svm->priv->dbg),
+                           svm->priv->corefile_unloaded_sig);
 	gtk_signal_disconnect (GTK_OBJECT (svm->priv->dbg),
 						   svm->priv->execution_exited_sig);
     gtk_signal_disconnect (GTK_OBJECT (svm->priv->dbg),
@@ -842,6 +854,14 @@ execution_source_line_cb (GdfDebuggerClient *dbg,
 	svm->priv->prev_sv = GDF_SOURCE_VIEWER (sv);
 }
 
+void
+corefile_unloaded_cb (GdfDebuggerClient *dbg, GdfSourceViewerManager *svm)
+{
+    if (svm->priv->prev_sv)
+        gdf_source_viewer_clear_current_line (svm->priv->prev_sv);
+    svm->priv->prev_sv = NULL;
+}
+
 void
 execution_exited_cb (GdfDebuggerClient *dbg, gint exit_code,
                      GdfSourceViewerManager *svm)
@@ -859,10 +879,11 @@ frame_changed_cb (GdfDebuggerClient *dbg, gint new_frame,
     GDF_StackFrame *frame = NULL;
     GtkWidget *sv;
     char *full_path = NULL;
+    GdfDebuggerClientResult res;
     
-    gdf_debugger_client_get_frame (dbg, new_frame, &frame);
+    res = gdf_debugger_client_get_frame (dbg, new_frame, &frame);
     
-    if (frame->location.file[0]) {
+    if (res == GDF_DEBUGGER_CLIENT_OK && frame->location.file[0]) {
         gdf_debugger_client_get_absolute_source_path (dbg, 
                                                       frame->location.file,
                                                       &full_path);
@@ -873,8 +894,11 @@ frame_changed_cb (GdfDebuggerClient *dbg, gint new_frame,
         gdf_source_viewer_view_line (GDF_SOURCE_VIEWER (sv), 
                                      frame->location.line);
         g_free (full_path);
-    } else {
+    } else if (res == GDF_DEBUGGER_CLIENT_OK) {
         create_nonexistent_page (svm, frame->function);
+    } else {
+        g_warning ("Source viewer could not get stack.");
+        create_nonexistent_page (svm, "Unknown function");
     }
     
     CORBA_free (frame);
