@@ -498,6 +498,7 @@ scroll_to_line (GdfSourceViewer *sv, gint line_num)
     adjustment = 
         gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sv->priv->scrolled));
     gtk_adjustment_set_value (adjustment, (gfloat) pixel);
+    gtk_widget_queue_draw (sv->priv->clist);
 }
 
 static void
@@ -578,7 +579,7 @@ do_popup (GdfSourceViewer *sv, SVBreakpoint *bp, gint row)
         popup_menu[0].hint = _("Enable the selected breakpoint");
         popup_menu[0].moreinfo = (gpointer)enable_breakpoint_selected_cb;
         popup_menu[0].pixmap_type = GNOME_APP_PIXMAP_STOCK;
-        popup_menu[0].pixmap_info = GNOME_STOCK_PIXMAP_STOP;
+        popup_menu[0].pixmap_info = GNOME_STOCK_MENU_STOP;
     } 
 	
     if (bp) {
