@@ -30,6 +30,8 @@ struct _GdfStackBrowserPriv {
     GdfDebuggerClient *dbg;
 
     int frame_changed_sig;
+    int corefile_loaded_sig;
+    int corefile_unloaded_sig;
     int execution_stopped_sig;
     int execution_exited_sig;
     int execution_killed_sig;
@@ -56,7 +58,6 @@ static void frame_changed_cb (GdfDebuggerClient *dbg, int frame_id,
                               GdfStackBrowser *sb);
 static void execution_stopped_cb (GdfDebuggerClient *dbg,
                                   GdfStackBrowser *sb);
-
 static void select_row_cb (GtkWidget *clist, gint row, gint column, 
                            GdkEvent *event, GdfStackBrowser *sb);
 static void create_children (GdfStackBrowser *sb);
@@ -215,6 +216,16 @@ connect_debugger_signals (GdfStackBrowser *sb)
                             "stack_frame_changed",
                             GTK_SIGNAL_FUNC (frame_changed_cb),
                             (gpointer)sb);
+    sb->priv->corefile_loaded_sig = 
+        gtk_signal_connect (GTK_OBJECT (sb->priv->dbg), 
+                            "corefile_loaded",
+                            GTK_SIGNAL_FUNC (execution_stopped_cb),
+                            (gpointer)sb);
+    sb->priv->corefile_unloaded_sig = 
+        gtk_signal_connect_object (GTK_OBJECT (sb->priv->dbg),
+                                   "corefile_unloaded",
+                                   GTK_SIGNAL_FUNC (gtk_clist_clear),
+                                   (gpointer)sb->priv->clist);
     sb->priv->execution_stopped_sig = 
         gtk_signal_connect (GTK_OBJECT (sb->priv->dbg), 
                             "execution_stopped",
@@ -240,10 +251,21 @@ connect_debugger_signals (GdfStackBrowser *sb)
 void
 disconnect_debugger_signals (GdfStackBrowser *sb)
 {
+    gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg),
+                           sb->priv->frame_changed_sig);
+    gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg),
+                           sb->priv->corefile_loaded_sig);
+    gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg),
+                           sb->priv->corefile_unloaded_sig);
     gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg),
                            sb->priv->execution_stopped_sig);
     gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg), 
                            sb->priv->execution_exited_sig);
+    gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg), 
+                           sb->priv->execution_killed_sig);
+    gtk_signal_disconnect (GTK_OBJECT (sb->priv->dbg), 
+                           sb->priv->program_unloaded_sig);
+    
 }
 
 void
