@@ -616,7 +616,8 @@ add_current_expression (GdfVariableViewer *vv)
     
     if (strcmp (text, "")) {
         gdf_symbol_set_client_add_expression (vv->priv->ss, 
-                                              text);
+                                              text,
+                                              NULL);
     }
 
     gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (vv->priv->entry))), "");
