@@ -26,6 +26,7 @@
 #include "gdf-tools.h"
 
 #include <liboaf/liboaf.h>
+#include <libgnome/gnome-util.h>
 #include "gdf-event-marshallers.h"
 
 static void debugger_client_class_init (GdfDebuggerClientClass *class);
@@ -55,6 +56,8 @@ enum {
     BREAKPOINT_DELETED,
     ATTACHED,
     DETACHED,
+    COREFILE_LOADED,
+    COREFILE_UNLOADED,
     EXECUTION_STARTED,
     EXECUTION_RUNNING,
     EXECUTION_STOPPED,
@@ -71,7 +74,7 @@ static gint debugger_client_signals[LAST_SIGNAL];
 
 static void 
 gdf_debugger_client_construct (GdfDebuggerClient *dbg, 
-                                CORBA_Object corba_object)
+                               CORBA_Object corba_object)
 {
     CORBA_Object channel;
     CORBA_Environment ev;
@@ -100,6 +103,17 @@ gdf_debugger_client_construct (GdfDebuggerClient *dbg,
                         (gpointer)dbg);  
 }
 
+/**
+ * gdf_debugger_client_new_for_mime_type:
+ * @type: The mime-type the debugger should be capable of debugging.
+ * 
+ * Creates a #GdfDebuggerClient object.  This function queries the OAF database
+ * to find a suitable backend for debugging the given mime-type.  The first
+ * backend found will be activated.  If a suitable backend cannot be found, 
+ * the function will return NULL.
+ *
+ * Return value: A newly created #GdfDebuggerClient object.
+ **/
 GdfDebuggerClient *
 gdf_debugger_client_new_for_mime_type (const char *type)
 {
@@ -130,12 +144,12 @@ gdf_debugger_client_new_for_mime_type (const char *type)
 
 /**
  * gdf_debugger_client_new:
- * @iid: Backend to load.
+ * @iid: OAF IID of the backend to activate.
  *
- * Starts a new GDF::Debugger server and returns a GdfDebuggerClient
- * with a reference to that object.
+ * Creates a #GdfDebuggerClient object.  This object activates a new 
+ * GDF::Debugger server with the given OAF IID.
  *
- * Returns: the initialized GdfDebuggerClient object.
+ * Returns: A newly-created #GdfDebuggerClient object.
  */
 GdfDebuggerClient *
 gdf_debugger_client_new (const char *iid)
@@ -153,7 +167,7 @@ gdf_debugger_client_new (const char *iid)
     CORBA_exception_init (&ev);
 
     corba_object = oaf_activate_from_id ((const OAF_ActivationID)iid, 
-                                         OAF_FLAG_IGNORE_EXISTING, 
+                                         OAF_FLAG_PRIVATE, 
                                          NULL, &ev);    
     
     if (CORBA_Object_is_nil (corba_object, &ev)) {
@@ -171,14 +185,17 @@ gdf_debugger_client_new (const char *iid)
 
 
 /**
- * gdf_debugger_client_new_from_objref:
- * @program: GDF::Program CORBA object reference.
- *
- * Creates a GdfDebuggerClient GTK Object which wraps around the
- * GDF::Program CORBA object.
- *
- * Returns: the initialized GdfDebuggerClient object.
- */
+ * gdf_debugger_client_new_from_corba:
+ * @debugger: A #GDF::Debugger CORBA object.
+ * 
+ * Creates a GdfDebuggerClient object.  This object will be used to manipulate
+ * the backend provided by the @debugger argument.
+ * 
+ * The @debugger object will not be ref'ed.  The caller must ref the object
+ * before passing in the object.
+ * 
+ * Return value: A newly-created GdfDebuggerClient object.
+ **/
 GdfDebuggerClient *
 gdf_debugger_client_new_from_corba (GDF_Debugger debugger) 
 {
@@ -192,7 +209,7 @@ gdf_debugger_client_new_from_corba (GDF_Debugger debugger)
    
     CORBA_exception_init (&ev);
 
-    Bonobo_Unknown_ref (debugger, &ev);
+    //Bonobo_Unknown_ref (debugger, &ev);
 
     /* FIXME: Check env */
     
@@ -208,7 +225,7 @@ gdf_debugger_client_get_type (void)
     
     if (!type) {
         GtkTypeInfo info = {
-            "Handle to a GDF::Debugger object",
+            "GdfDebuggerClient",
             sizeof (GdfDebuggerClient),
             sizeof (GdfDebuggerClientClass),
             (GtkClassInitFunc) debugger_client_class_init,
@@ -224,6 +241,33 @@ gdf_debugger_client_get_type (void)
     return type;
 }
 
+/**
+ * gdf_debugger_client_get_state:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Gets the current state of the debugger.  The state is a bitfield with 
+ * the following values defined:
+ *         
+ * %GDF_Debugger_BINARY_LOADED - Whether a binary has been loaded into
+ *     the debugger.
+ * 
+ * %GDF_Debugger_TARGET_EXISTS - Whether a process that can be run,
+ *     stepped through, etc. exists.
+ *
+ * %GDF_Debugger_TARGET_EXECUTING - Whether the target is in the 
+ *     process of executing code.  %GDF_Debugger_TARGET_EXISTS must be  
+ *     set for this flag to be set.
+ *
+ * %GDF_Debugger_TARGET_ATTACHED - Whether the debugger is attached to
+ *     an external process.  %GDF_Debugger_TARGET_EXISTS must be set for
+ *     this flag to be set.
+ *
+ * %GDF_Debugger_COREFILE - Whether the debugger has a corefile loaded.
+ *     %GDF_Debugger_COREFILE and GDF_Debugger_TARGET_EXISTS are mutually
+ *     exclusive.
+ * 
+ * Return value: A #GDF_Debugger_StateFlag value.
+ **/
 GDF_Debugger_StateFlag
 gdf_debugger_client_get_state (GdfDebuggerClient *dbg)
 {
@@ -244,6 +288,17 @@ gdf_debugger_client_get_state (GdfDebuggerClient *dbg)
     return ret;
 }
 
+/**
+ * gdf_debugger_client_load_binary:
+ * @dbg: The G#dfDebuggerClient object.
+ * @file_name: The name of the binary to laod into the debugger.
+ * 
+ * Loads a binary file into the debugger.  If this function call is sucessful,
+ * the %GDF_Debugger_BINARY_LOADED state flag will be set.
+ *
+ * Return value: A #GdfDebuggerClientResult indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_load_binary (GdfDebuggerClient *dbg, 
                                  char *file_name)
@@ -267,6 +322,17 @@ gdf_debugger_client_load_binary (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_unload_binary:
+ * @dbg: The $GdfDebuggerClient object.
+ * 
+ * Unloads the binary file in the debugger.  The %GDF_Debugger_BINARY_LOADED
+ * flag must be set for this operation to succeed, and will be unset on 
+ * success.
+ * 
+ * Return value: A $GdfDebuggerClientResult indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult 
 gdf_debugger_client_unload_binary (GdfDebuggerClient *dbg)
 {
@@ -287,6 +353,20 @@ gdf_debugger_client_unload_binary (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_sources:
+ * @dbg: The $GdfDebuggerClient object.
+ * @ret: An out parameter, through which a #GDF_source_file_list will be
+ *       returned.  The list will be NULL if the operation is unsuccessful.
+ *       If successful it must be freed by the client using CORBA_free().
+ * 
+ * Gets a list of source files from the debugger.  
+ * The %GDF_Debugger_BINARY_LOADED flag must be set for this operation to 
+ * succeed.
+ * 
+ * Return value: A #GdfDebuggerClientResult indicating the success of the 
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_sources (GdfDebuggerClient *dbg, 
                                  GDF_source_file_list **ret)
@@ -314,6 +394,20 @@ gdf_debugger_client_get_sources (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_absolute_source_path:
+ * @dbg: The #GdfDebuggerClient object.
+ * @file_name: The file name to be made absolute.
+ * @ret: An out parameter, through which the absolute source path will be 
+ *       returned.  It will be NULL if the operation fails.  If successful 
+ *       the path must be freed by the caller using g_free().
+ * 
+ * Converts a filename into an absolute source path, if possible locating the
+ * source file from which the @file_name was compiled.
+ * 
+ * Return value: A #GdfDebuggerClientResult indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_absolute_source_path (GdfDebuggerClient *dbg,
                                               const char *file_name,
@@ -351,36 +445,57 @@ gdf_debugger_client_get_absolute_source_path (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_execute:
+ * @dbg: The #GdfDebuggerClient object.
+ * @args: The arguments to pass on the command line.
+ * 
+ * Begins execution of the program, passing @args on the command line.  This 
+ * function will return immediately, and signals will be used to communicate 
+ * state changes in the executing program.
+ *
+ * This function requires the %GDF_Debugger_BINARY_LOADED flag to be set,
+ * and for the 5GDF_Debugger_TARGET_EXISTS and %GDF_Debugger_COREFILE flags
+ * to be cleared.  This function will set the %GDF_Debugger_TARGET_EXISTS and
+ * %GDF_Debugger_TARGET_EXECUTING flags.
+ * 
+ * Return value: A #GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
-gdf_debugger_client_execute (GdfDebuggerClient *dbg, 
-                             int argc,
-                             char *argv[])
+gdf_debugger_client_execute (GdfDebuggerClient *dbg,
+                             const char *args)
 {
-    GDF_arg_list *arg_list;
     CORBA_Environment ev;    
     GdfDebuggerClientResult res;
 
     g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
-    g_return_val_if_fail (argv != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
                           GDF_DEBUGGER_CLIENT_BAD_PARAMS);
 
     CORBA_exception_init (&ev);
     
-    /* FIXME: This isn't right */
-    arg_list = GDF_arg_list__alloc();
-    arg_list->_length = argc;
-    arg_list->_buffer = (CORBA_char**)argv;
-    
-    GDF_Debugger_execute (dbg->objref,
-                          arg_list,
-                          &ev);
+    GDF_Debugger_execute (dbg->objref, args ? args : "", &ev);
 
     res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
     return res;
 }
 
+/**
+ * gdf_debugger_client_attach:
+ * @dbg: The #GdfDebuggerClient object.
+ * @pid: The process id to attach to.
+ * 
+ * Attaches the debugger to an existing process.  This function requires the
+ * %GDF_Debugger_BINARY_LOADED flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXISTS and %GDF_Debugger_COREFILE flags to be cleared.
+ * If successful, this function will set the %GDF_Debugger_TARGET_EXISTS and 
+ * %GDF_Debugger_ATTACHED flags.
+ * 
+ * Return value: A #GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_attach (GdfDebuggerClient *dbg, int pid)
 {
@@ -400,6 +515,18 @@ gdf_debugger_client_attach (GdfDebuggerClient *dbg, int pid)
     return res;
 }
 
+/**
+ * gdf_debugger_client_detach:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Detaches from an external process.  This functino requires the 
+ * %GDF_Debugger_ATTACHED flag to be set.  If successful, the 
+ * %GDF_Debugger_ATTACHED and %GDF_Debugger_TARGET_EXISTS flags will be 
+ * cleared.
+ * 
+ * Return value: A #GdfDebuggerClientResult value indicating the success of 
+ *               the operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_detach (GdfDebuggerClient *dbg)
 {
@@ -419,12 +546,26 @@ gdf_debugger_client_detach (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_load_corefile:
+ * @dbg: The #GdfDebuggerClient object.
+ * @corefile_name: Filename of the corefile to load.
+ * 
+ * Loads a corefile into the debugger.  This function requires the 
+ * %GDF_Debugger_BINARY_LOADED flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXISTS and %GDF_Debugger_COREFILE flags to be cleared.
+ * Upon success, the %GDF_Debugger_COREFILE flag will be set.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_load_corefile (GdfDebuggerClient *dbg,
                                    const char *corefile_name)
 {
     CORBA_Environment ev;
     GdfDebuggerClientResult res;
+    char *path = NULL;
 
     g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     g_return_val_if_fail (corefile_name != NULL, 
@@ -432,17 +573,113 @@ gdf_debugger_client_load_corefile (GdfDebuggerClient *dbg,
     g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
                           GDF_DEBUGGER_CLIENT_BAD_PARAMS);
     
+    if (g_path_is_absolute (corefile_name)) {
+        path = g_strdup (corefile_name);
+    } else {
+        char *dir = g_get_current_dir ();
+        path = g_concat_dir_and_file (dir, corefile_name);
+        g_free (dir);
+    }
     CORBA_exception_init (&ev);
     
     GDF_Debugger_load_corefile (dbg->objref,
-                                (CORBA_char*)corefile_name,
+                                (CORBA_char*)path,
                                 &ev);
     
+    g_free (path);
+
+    res = exception_to_result (&ev);
+    CORBA_exception_free (&ev);
+    return res;
+}
+
+/**
+ * gdf_debugger_client_unload_corefile:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Unloads a corefile from the the debugger.  This function requires the
+ * %GDF_Debugger_COREFILE flag to be set.  
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
+GdfDebuggerClientResult 
+gdf_debugger_client_unload_corefile (GdfDebuggerClient *dbg)
+{
+    CORBA_Environment ev;
+    GdfDebuggerClientResult res;
+
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg),
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+
+    CORBA_exception_init (&ev);
+
+    GDF_Debugger_unload_corefile (dbg->objref, &ev);
+    
     res = exception_to_result (&ev);
     CORBA_exception_free (&ev);
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_breakpoints:
+ * @dbg: The #GdfDebuggerClient object.
+ * @ret: An out parameter through which a GDF_BreakpointList containing all 
+ *       the breakpoints set will be returned.  If the operation is 
+ *       unsuccessful this value will be NULL.  If successful, it must be 
+ *       freed by the caller using CORBA_free().
+ * 
+ * Gets a list of all the breakpoints set in the debugger.  This function
+ * requires the %GDF_Debugger_BINARY_LOADED flag to be set. 
+ *
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
+GdfDebuggerClientResult
+gdf_debugger_client_get_breakpoints (GdfDebuggerClient *dbg,
+                                     GDF_BreakpointList **ret)
+{
+    CORBA_Environment ev;
+    GdfDebuggerClientResult res;
+    
+    g_return_val_if_fail (dbg != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (GDF_IS_DEBUGGER_CLIENT (dbg), 
+                          GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+    g_return_val_if_fail (ret != NULL, GDF_DEBUGGER_CLIENT_BAD_PARAMS);
+
+    CORBA_exception_init (&ev);
+    *ret = GDF_Debugger_get_breakpoints (dbg->objref, &ev);
+    
+    res = exception_to_result (&ev);
+    if (res != GDF_DEBUGGER_CLIENT_OK) {
+        *ret = NULL;
+    }
+
+    CORBA_exception_free (&ev);
+    return res;
+}
+
+/**
+ * gdf_debugger_client_set_breakpoint:
+ * @dbg: The #GdfDebuggerClient object.
+ * @file_name: The file to set the breakpoint in.  The path should not be
+ *             absolute, the debugging information will be used to find
+ *             the correct file.
+ * @line_num: The line number to set the breakpoint on.
+ * @condition: An expression in the target language that will be evaludated 
+ *             by the debugger when the location is reached.  Execution will 
+ *             only stop if the expression evaluates true.  NULL indicates an
+ *             unconditional breakpoint.
+ * @bp_num: An out parameter through which the breakpoint number will be 
+ *          returned.  Will be -1 if the operation was unsuccessful.
+ * 
+ * Sets a breakpoint at the given location.  Execution will stop when
+ * the location is reached, assuming @condition evaluates to %TRUE.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_set_breakpoint (GdfDebuggerClient *dbg,
                                     const char *file_name,
@@ -481,6 +718,27 @@ gdf_debugger_client_set_breakpoint (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_set_breakpoint_function:
+ * @dbg: 
+ * @file_name: The file to set the breakpoint in.  The path should not be
+ *             absolute, the debugging information will be used to find the
+ *             correct file.  This can be NULL, in which case only the function
+ *             name will be used.
+ * @function_name: The function to set the breakpoint on.
+ * @condition: An expression in the target language that will be evaludated 
+ *             by the debugger when the location is reached.  Execution will 
+ *             only stop if the expression evaluates true. NULL indicates an
+ *             unconditional breakpoint.
+ * @bp_num: An out parameter through which the breakpoint number will be 
+ *          returned.  Will be -1 if the operation was unsuccessful.
+ * 
+ * Sets a breakpoint on a given function.  Execution will stop when execution
+ * enters the given function, assuming @condition evaluates to %TRUE.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_set_breakpoint_function (GdfDebuggerClient *dbg,
                                              const char *file_name,
@@ -521,6 +779,25 @@ gdf_debugger_client_set_breakpoint_function (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_set_watchpoint:
+ * @dbg: The #GdfDebuggerClient object.
+ * @expr: The expression to evaluate.
+ * @condition: An expression in the target language that will be evaludated 
+ *             by the debugger when the location is reached.  Execution will 
+ *             only stop if the expression evaluates true. NULL indicates an
+ *             unconditional breakpoint.
+ * @bp_num: An out parameter through which the breakpoint number will be 
+ *          returned.  Will be -1 if the operation was unsuccessful.
+ * 
+ * Sets a watchpoint (value-based breakpoint) on the given expression.  
+ * Execution will stop when the value of the expression changes, assuming 
+ * @condition evaluates to %TRUE.  The watchpoint can be manipulated using
+ * the breakpoint manipulation functions.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_set_watchpoint (GdfDebuggerClient *dbg,
                                     const char *expr,
@@ -556,6 +833,16 @@ gdf_debugger_client_set_watchpoint (GdfDebuggerClient *dbg,
 }
 
 
+/**
+ * gdf_debugger_client_enable_breakpoint:
+ * @dbg: The #GdfDebuggerClient object.
+ * @bp_num: The breakpoint to enable.
+ * 
+ * Enables a breakpoint.  
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
                                        long bp_num)
@@ -579,6 +866,16 @@ gdf_debugger_client_enable_breakpoint (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_disable_breakpoint:
+ * @dbg: The #GdfDebuggerClient object.
+ * @bp_num: The breakpoint to disable.
+ * 
+ * Disables a breakpoint.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_disable_breakpoint (GdfDebuggerClient *dbg,
                                         long bp_num)
@@ -600,6 +897,17 @@ gdf_debugger_client_disable_breakpoint (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_delete_breakpoint:
+ * @dbg: The #GdfDebuggerClient object.
+ * @bp_num: The breakpoint to delete.
+ * 
+ * Deletes a breakpoint.  When this call returns, @bp_num is no longer a valid
+ * breakpoint identifier.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_delete_breakpoint (GdfDebuggerClient *dbg,
                                        long bp_num)
@@ -621,6 +929,19 @@ gdf_debugger_client_delete_breakpoint (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_breakpoint_info:
+ * @dbg: The #GdfDebuggerClient object.
+ * @bp_num: The breakpoint to get information about.
+ * @ret: An out parameter through which a GDF_Breakpoint structure will be
+ *       returned.  This value will be NULL if the operation failed.  If
+ *       successful the caller must free the structure using CORBA_free().
+ * 
+ * Retrieves information about a given breakpoint.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_breakpoint_info (GdfDebuggerClient *dbg,
                                          long bp_num,
@@ -648,6 +969,21 @@ gdf_debugger_client_get_breakpoint_info (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_registers:
+ * @dbg: The #GdfDebuggerClient object.
+ * @floating_pt: Whether floating point registers should be returned.
+ * @ret: An out parameter through which a GDF_RegisterList value will be
+ *       returned.  If the operation is unsuccessful this value will be NULL.
+ *       If successful, the value must be freed using CORBA_free().
+ * 
+ * Gets the values of the registers.  This is returned as a GDF_RegisterList,
+ * which is a CORBA sequence of GDF_Register values.  A GDF_Register is a
+ * structure containing two strings, the register name and the register value.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_registers (GdfDebuggerClient *dbg,
                                    gboolean floating_pt,
@@ -675,6 +1011,15 @@ gdf_debugger_client_get_registers (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_current_frame:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Gets the number of the currently selected stack frame.
+ * 
+ * Return value: The number of the currently selected stack frame, or -1 if 
+ *               there is no stack.
+ **/
 long
 gdf_debugger_client_current_frame (GdfDebuggerClient *dbg)
 {
@@ -683,19 +1028,47 @@ gdf_debugger_client_current_frame (GdfDebuggerClient *dbg)
 
     return dbg->priv->cur_frame;
 }
- 
+
+/**
+ * gdf_debugger_client_up_frame:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Selects the stack frame above the currently selected frame.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_up_frame (GdfDebuggerClient *dbg)
 {
     return gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame + 1);
 }
 
+/**
+ * gdf_debugger_client_down_frame:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Selects the stack frame below the currently selected frame.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_down_frame (GdfDebuggerClient *dbg)
 {
     return gdf_debugger_client_change_frame (dbg, dbg->priv->cur_frame - 1);
 }
 
+/**
+ * gdf_debugger_client_change_frame:
+ * @dbg: The #GdfDebuggerClient object.
+ * @new_id: The frame to select.
+ * 
+ * Changes the currently selected stack frame.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation. 
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_change_frame (GdfDebuggerClient *dbg, 
                                   long new_id)
@@ -716,6 +1089,19 @@ gdf_debugger_client_change_frame (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_frame:
+ * @dbg: The #GdfDebuggerClient object.
+ * @id: The stack frame to retrieve.
+ * @ret: A GDF_StackFrame value describing the stack frame indicated 
+ *       by @frame.  This value will be NULL if the operation is unsuccessful,
+ *       and must be freed by CORBA_free() if successful.
+ * 
+ * Retrieves information about a given stack frame.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_frame (GdfDebuggerClient *dbg,
                                long id,
@@ -743,6 +1129,19 @@ gdf_debugger_client_get_frame (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_backtrace:
+ * @dbg: The #GdfDebuggerClient object.
+ * @ret: A GDF_Stack value containing a sequnce of stack frames in the 
+ *       currently executing program.  This value will be NULL if the 
+ *       operation is unsuccessful, and must be freed by CORBA_free() if 
+ *       successful.
+ * 
+ * Retrieves the stack as a CORBA sequence of GDF_StackFrame values.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_backtrace (GdfDebuggerClient *dbg,
                                    GDF_Stack **ret)
@@ -768,6 +1167,19 @@ gdf_debugger_client_get_backtrace (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_cont:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Continue execution of a stopped program.  This operation requires the 
+ * %GDF_Debugger_TARGET_EXISTS flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXECUTING flag to be cleared.  The 
+ * %GDF_Debugger_TARGET_EXECUTING flag will be set upon returning from the 
+ * function.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_cont (GdfDebuggerClient *dbg)
 {
@@ -787,6 +1199,19 @@ gdf_debugger_client_cont (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_stop:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Ends execution of a program.  This operation requires the 
+ * %GDF_Debugger_TARGET_EXISTS flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXECUTING flag to be cleared.  The 
+ * %GDF_Debugger_TARGET_EXISTS flag will be cleared up returning from the 
+ * function.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_stop (GdfDebuggerClient *dbg)
 {
@@ -805,6 +1230,18 @@ gdf_debugger_client_stop (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_restart:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Restarts execution of the program. This operation requires the 
+ * %GDF_Debugger_TARGET_EXISTS flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXECUTING flag to be cleared. The flags will remain
+ * unchanged upon return.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_restart (GdfDebuggerClient *dbg)
 {
@@ -824,6 +1261,19 @@ gdf_debugger_client_restart (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_step_over:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Executes a single source line, not entering any function calls. 
+ * Restarts execution of the program. This operation requires the 
+ * %GDF_Debugger_TARGET_EXISTS flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXECUTING flag to be cleared. The 
+ * %GDF_Debugger_TARGET_EXECUTING flag will be set upon return.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_step_over (GdfDebuggerClient *dbg)
 {
@@ -843,6 +1293,19 @@ gdf_debugger_client_step_over (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_step_into:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Executes a single source line, entering any function calls. 
+ * Restarts execution of the program. This operation requires the 
+ * %GDF_Debugger_TARGET_EXISTS flag to be set, and the 
+ * %GDF_Debugger_TARGET_EXECUTING flag to be cleared. The 
+ * %GDF_Debugger_TARGET_EXECUTING flag will be set upon return. 
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_step_into (GdfDebuggerClient *dbg)
 {
@@ -862,6 +1325,19 @@ gdf_debugger_client_step_into (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_step_out:
+ * @dbg: The #GdfDebuggerClient object.
+ * 
+ * Executes the program until it returns from the current stack frame.
+ * This operation requires the  %GDF_Debugger_TARGET_EXISTS flag to be set, 
+ * and the %GDF_Debugger_TARGET_EXECUTING flag to be cleared. The 
+ * %GDF_Debugger_TARGET_EXECUTING flag will be set upon return. 
+
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_step_out (GdfDebuggerClient *dbg)
 {
@@ -881,6 +1357,18 @@ gdf_debugger_client_step_out (GdfDebuggerClient *dbg)
     return res;
 }
 
+/**
+ * gdf_debugger_client_get_locals:
+ * @dbg: The #GdfDebuggerClient object.
+ * @ret: An out parameter through which a GdfSymbolSetClient will be returned.
+ *       This value will be NULL if the operation is unsuccessful, and the
+ *       object must be destroyed with gtk_object_unref() if successful.
+ * 
+ * Retrieves a #GdfSymbolSetClient object representing the local variables.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_get_locals (GdfDebuggerClient *dbg,
                                 GdfSymbolSetClient **ret)
@@ -909,6 +1397,19 @@ gdf_debugger_client_get_locals (GdfDebuggerClient *dbg,
     return res;
 }   
 
+/**
+ * gdf_debugger_client_allocate_symbol_set:
+ * @dbg: The #GdfDebuggerClient object.
+ * @ret: An out parameter through which a GdfSymbolSetClient will be returned.
+ *       This value will be NULL if the operation is unsuccessful, and the
+ *       object must be destroyed with gtk_object_unref() if successful. 
+ * 
+ * Retrieves a GdfSymbolSetClient object representing a newly allocated 
+ * symbol set.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_allocate_symbol_set (GdfDebuggerClient *dbg,
                                          GdfSymbolSetClient **ret)
@@ -938,6 +1439,16 @@ gdf_debugger_client_allocate_symbol_set (GdfDebuggerClient *dbg,
     return res;
 }
 
+/**
+ * gdf_debugger_client_set_output_tty:
+ * @dbg: The #GdfDebuggerClient object.
+ * @tty_name: The terminal device that should be used for program interaction.
+ * 
+ * Sets the terminal device used for program interaction to @tty_name.
+ * 
+ * Return value: A GdfDebuggerClientResult value indicating the success of the
+ *               operation.  
+ **/
 GdfDebuggerClientResult
 gdf_debugger_client_set_output_tty (GdfDebuggerClient *dbg,
                                     const char *tty_name)
@@ -1048,6 +1559,22 @@ debugger_client_class_init (GdfDebuggerClientClass *klass)
                         gtk_marshal_NONE__NONE,
                         GTK_TYPE_NONE, 0,
                         GTK_TYPE_NONE);
+    debugger_client_signals [COREFILE_LOADED] = 
+        gtk_signal_new ("corefile_loaded",
+                        GTK_RUN_FIRST,
+                        object_class->type,
+                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
+                                           corefile_loaded),
+                        gtk_marshal_NONE__NONE,
+                        GTK_TYPE_NONE, 0);
+    debugger_client_signals [COREFILE_UNLOADED] = 
+        gtk_signal_new ("corefile_unloaded",
+                        GTK_RUN_FIRST,
+                        object_class->type,
+                        GTK_SIGNAL_OFFSET (GdfDebuggerClientClass,
+                                           corefile_unloaded),
+                        gtk_marshal_NONE__NONE,
+                        GTK_TYPE_NONE, 0);
     debugger_client_signals [EXECUTION_STARTED] = 
         gtk_signal_new ("execution_started",
                         GTK_RUN_FIRST,
@@ -1208,6 +1735,12 @@ emit_event_signal (GdfDebuggerClient *dbg,
     } else if (strcmp (event->event_name, "detached") == 0) {
         gtk_signal_emit (GTK_OBJECT (dbg),
                          debugger_client_signals[DETACHED]);
+    } else if (strcmp (event->event_name, "corefile_loaded") == 0) {
+        gtk_signal_emit (GTK_OBJECT (dbg),
+                         debugger_client_signals[COREFILE_LOADED]);
+    } else if (strcmp (event->event_name, "corefile_unloaded") == 0) {
+        gtk_signal_emit (GTK_OBJECT (dbg),
+                         debugger_client_signals[COREFILE_UNLOADED]);
     } else if (strcmp (event->event_name, "started") == 0) {
         gtk_signal_emit (GTK_OBJECT (dbg),
                          debugger_client_signals[EXECUTION_STARTED]);
@@ -1280,20 +1813,16 @@ exception_to_result (CORBA_Environment *ev)
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NO_SYMBOLS));
         g_hash_table_insert (table, ex_GDF_Debugger_DoesntExist,
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_DOESNT_EXIST));
-        g_hash_table_insert (table, ex_GDF_Debugger_NoBinary,
-                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NO_BINARY));
         g_hash_table_insert (table, ex_GDF_Debugger_NotExecutable,
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_EXECUTABLE));
-        g_hash_table_insert (table, ex_GDF_Debugger_NotAttached,
-                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_ATTACHED));
-        g_hash_table_insert (table, ex_GDF_Debugger_NotRunning,
-                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_NOT_RUNNING));
         g_hash_table_insert (table, ex_GDF_Debugger_SecurityException,
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_SECURITY_EXCEPTION));
         g_hash_table_insert (table, ex_GDF_Debugger_InvalidState,
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_INVALID_STATE));
         g_hash_table_insert (table, ex_GDF_Debugger_UnknownError,
                              GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_UNKNOWN_ERROR));
+        g_hash_table_insert (table, ex_GDF_Debugger_InvalidCorefile,
+                             GINT_TO_POINTER (GDF_DEBUGGER_CLIENT_INVALID_COREFILE));
     }
 
     res = GDF_DEBUGGER_CLIENT_OK;
