@@ -64,6 +64,8 @@ struct _GdfDebuggerClientClass {
     void (*breakpoint_deleted) (GdfDebuggerClient *, gint bp_num);
     void (*attached) (GdfDebuggerClient *);
     void (*detached) (GdfDebuggerClient *);
+    void (*corefile_loaded) (GdfDebuggerClient *);
+    void (*corefile_unloaded) (GdfDebuggerClient *);
     void (*execution_started) (GdfDebuggerClient *);
     void (*execution_running) (GdfDebuggerClient *);
     void (*execution_stopped) (GdfDebuggerClient *);
@@ -85,13 +87,11 @@ enum _GdfDebuggerClientResult {
     GDF_DEBUGGER_CLIENT_INVALID_BINARY,
     GDF_DEBUGGER_CLIENT_NO_SYMBOLS,
     GDF_DEBUGGER_CLIENT_DOESNT_EXIST,
-    GDF_DEBUGGER_CLIENT_NO_BINARY,
     GDF_DEBUGGER_CLIENT_NOT_EXECUTABLE,
-    GDF_DEBUGGER_CLIENT_NOT_ATTACHED,
-    GDF_DEBUGGER_CLIENT_NOT_RUNNING,
     GDF_DEBUGGER_CLIENT_SECURITY_EXCEPTION,
     GDF_DEBUGGER_CLIENT_INVALID_STATE,
     GDF_DEBUGGER_CLIENT_BAD_PARAMS,
+    GDF_DEBUGGER_CLIENT_INVALID_COREFILE,
     GDF_DEBUGGER_CLIENT_UNKNOWN_ERROR
 };
 
@@ -106,19 +106,20 @@ GdfDebuggerClientResult gdf_debugger_client_load_binary (GdfDebuggerClient *dbg,
 GdfDebuggerClientResult gdf_debugger_client_unload_binary (GdfDebuggerClient *dbg);
 
 GdfDebuggerClientResult gdf_debugger_client_execute (GdfDebuggerClient *dbg, 
-                                                     int argc,
-                                                     char *argv[]);
+                                                     const char *args);
 GdfDebuggerClientResult gdf_debugger_client_attach (GdfDebuggerClient *dbg, 
                                                     int pid);
 GdfDebuggerClientResult gdf_debugger_client_detach (GdfDebuggerClient *dbg);
 GdfDebuggerClientResult gdf_debugger_client_load_corefile (GdfDebuggerClient *dbg,
                                                            const char* corefile_name);
-
+GdfDebuggerClientResult gdf_debugger_client_unload_corefile (GdfDebuggerClient *dbg);
 GdfDebuggerClientResult gdf_debugger_client_get_sources (GdfDebuggerClient *dbg, 
                                                          GDF_source_file_list **ret);
 GdfDebuggerClientResult gdf_debugger_client_get_absolute_source_path (GdfDebuggerClient *dbg,
                                                                       const char *file_name,
                                                                       char **ret);
+GdfDebuggerClientResult gdf_debugger_client_get_breakpoints (GdfDebuggerClient *dbg,
+                                                             GDF_BreakpointList **ret);
 GdfDebuggerClientResult gdf_debugger_client_set_breakpoint (GdfDebuggerClient *dbg,
                                                             const char *file_name,
                                                             long line_num,
