@@ -26,6 +26,8 @@
 #include <gtk/gtkwidget.h>
 #include "gdf-debugger-client.h"
 
+char *gdf_parameters_dialog (void);
 GdfDebuggerClientResult gdf_attach_dialog (GdfDebuggerClient *dbg);
+void gdf_add_breakpoint_dialog (GdfDebuggerClient *dbg);
 
 #endif
